/*
To Do:
	* Add option to msg groups (by flag) if a specific player joins the server. - combine with family sharing to see if it is an alt account of the person.
	* Function for root admins to test how a note looks by using !testnote <steam id|flags>
	
	For moving files, have global boolean for when file is being moved - if attempting to loop files and bool is true, wait to loop and retry after short timer.
*/

#pragma semicolon 1
#define PLUGIN_VERSION "2.1"
#define TAG "[TOG Notes]"

#include <sourcemod>
#include <autoexecconfig>
#include <morecolors>

new	Handle:g_hFlagMode = INVALID_HANDLE;
new g_iFlagMode;
new	Handle:g_hPNMode = INVALID_HANDLE;
new g_iPNMode;
new	Handle:g_hTime = INVALID_HANDLE;
new Float:g_fTime;
new	Handle:g_hAddTime = INVALID_HANDLE;
new Float:g_fAddTime;
new Handle:g_hChatHex = INVALID_HANDLE;
new String:g_sChatHex[7];

public Plugin:myinfo =
{
	name = "TOG Notes",
	author = "That One Guy",
	description = "Server managers can leave notes for players or admin flag groups that appear X seconds after the player connects",
	version = PLUGIN_VERSION,
	url = "http://www.togcoding.com"
}

public OnPluginStart()
{
	AutoExecConfig_SetFile("tognotes");
	AutoExecConfig_CreateConVar("tognotes_version", PLUGIN_VERSION, "TOG Notes: Version", FCVAR_NOTIFY|FCVAR_DONTRECORD);
	
	g_hFlagMode = AutoExecConfig_CreateConVar("tognotes_flag_mode", "0", "Method for displaying admin flag notes: 0 = Chat, 1 = Menu, 2 = Both", FCVAR_NONE, true, 0.0, true, 2.0);
	HookConVarChange(g_hFlagMode, OnCVarChange);
	g_iFlagMode = GetConVarInt(g_hFlagMode);
	
	g_hPNMode = AutoExecConfig_CreateConVar("tognotes_pn_mode", "0", "Method for displaying personal notes: 0 = Chat, 1 = Menu, 2 = Both", FCVAR_NONE, true, 0.0, true, 2.0);
	HookConVarChange(g_hPNMode, OnCVarChange);
	g_iPNMode = GetConVarInt(g_hPNMode);
	
	g_hTime = AutoExecConfig_CreateConVar("tognotes_time", "10", "Time after player is authenticated to display message.", FCVAR_NONE, true, 0.0);
	HookConVarChange(g_hTime, OnCVarChange);
	g_fTime = GetConVarFloat(g_hTime);
	
	g_hAddTime = AutoExecConfig_CreateConVar("tognotes_addtime", "10", "Time between multiple messages.", FCVAR_NONE, true, 0.0);
	HookConVarChange(g_hAddTime, OnCVarChange);
	g_fAddTime = GetConVarFloat(g_hAddTime);
	
	g_hChatHex = CreateConVar("tognotes_chat_hex", "FF0000", "Default hex color to use for chat messages.", FCVAR_NONE);
	HookConVarChange(g_hChatHex, OnCVarChange);
	GetConVarString(g_hChatHex, g_sChatHex, sizeof(g_sChatHex));
	
	AutoExecConfig_ExecuteFile();
	AutoExecConfig_CleanFile();
}

public OnCVarChange(Handle:hCVar, const String:sOldValue[], const String:sNewValue[])
{
	if(hCVar == g_hFlagMode)
	{
		g_iFlagMode = StringToInt(sNewValue);
	}
	else if(hCVar == g_hPNMode)
	{
		g_iPNMode = StringToInt(sNewValue);
	}
	else if(hCVar == g_hChatHex)
	{
		GetConVarString(g_hChatHex, g_sChatHex, sizeof(g_sChatHex));
	}
	else if(hCVar == g_hTime)
	{
		g_fTime = GetConVarFloat(g_hTime);
	}
	else if(hCVar == g_hAddTime)
	{
		g_fAddTime = GetConVarFloat(g_hAddTime);
	}
}

public OnClientPostAdminCheck(client)
{
	decl String:sFolderPath[PLATFORM_MAX_PATH];
	BuildPath(Path_SM, sFolderPath, sizeof(sFolderPath), "configs/tognotes");
	if(DirExists(sFolderPath))
	{
		new Handle:hDirectory = INVALID_HANDLE;
		hDirectory = OpenDirectory(sFolderPath);
		decl String:sID[MAX_NAME_LENGTH];
		GetClientAuthString(client, sID, sizeof(sID));
		ReplaceString(sID, sizeof(sID), ":", "_");
		Format(sID, sizeof(sID), "%s.txt", sID);
		new Float:fCounter = 0.0;
		
		if(hDirectory != INVALID_HANDLE)
		{
			decl String:sFile[PLATFORM_MAX_PATH];
			while(ReadDirEntry(hDirectory, sFile, sizeof(sFile)))
			{
				//search for personal note
				if(StrEqual(sFile, sID, false))
				{
					new Float:fTime = g_fTime + (fCounter * g_fAddTime);
					fCounter = fCounter + 1.0;
					new Handle:data = CreateDataPack();
					WritePackCell(data, client);
					WritePackString(data, sFile);
					WritePackCell(data, 0);
					ResetPack(data);
					CreateTimer(fTime, TimerCallback_Note, data, TIMER_FLAG_NO_MAPCHANGE);
				}
				//search for notes left for admin groups that apply to player
				if(StrContains(sFile, "flags_", false) != -1)
				{
					decl String:sFlags[30];
					strcopy(sFlags, sizeof(sFlags), sFile);
					ReplaceString(sFlags, sizeof(sFlags), "flags_", "");
					if(StrContains(sFlags, "public", false) != -1)
					{
						new Float:fTime = g_fTime + (fCounter * g_fAddTime);
						fCounter = fCounter + 1.0;
						new Handle:data = CreateDataPack();
						WritePackCell(data, client);
						WritePackString(data, sFile);
						WritePackCell(data, 1);
						ResetPack(data);
						CreateTimer(fTime, TimerCallback_Note, data, TIMER_FLAG_NO_MAPCHANGE);
					}
					else if(HasFlags(client, sFlags))
					{
						new Float:fTime = g_fTime + (fCounter * g_fAddTime);
						fCounter = fCounter + 1.0;
						new Handle:data = CreateDataPack();
						WritePackCell(data, client);
						WritePackString(data, sFile);
						WritePackCell(data, 1);
						ResetPack(data);
						CreateTimer(fTime, TimerCallback_Note, data, TIMER_FLAG_NO_MAPCHANGE);
					}
				}
			}
		}
		CloseHandle(hDirectory);
	}
	else
	{
		LogError("Missing folder: %s", sFolderPath);
		ServerCommand("sm plugins unload tognotes.smx");
	}
}

public Action:TimerCallback_Note(Handle:hTimer, any:data)
{
	decl String:sFile[PLATFORM_MAX_PATH];
	new client = ReadPackCell(data);
	ReadPackString(data, sFile, sizeof(sFile));
	new iFlags = ReadPackCell(data);
	CloseHandle(data);
	if(!IsValidClient(client))
	{
		return Plugin_Continue;
	}
	decl String:sPath[PLATFORM_MAX_PATH];
	BuildPath(Path_SM, sPath, sizeof(sPath), "configs/tognotes/%s", sFile);
	if(iFlags) //admin flag note
	{
		if(g_iFlagMode == 0)
		{
			DisplayNote_Chat(client, sPath);
		}
		else if(g_iFlagMode == 1)
		{
			DisplayMenu(DisplayNote_Menu(client, iFlags, sPath), client, 0);
		}
		else if(g_iFlagMode == 2)
		{
			DisplayNote_Chat(client, sPath);
			DisplayMenu(DisplayNote_Menu(client, iFlags, sPath), client, 0);
		}
		else
		{
			LogError("Cvar 'tognotes_flag_mode' must be set to either 0, 1, or 2. Current value (%i) is invalid!", g_iFlagMode);
		}
	}
	else //personal note
	{
		if(g_iPNMode == 0)
		{
			DisplayNote_Chat(client, sPath);
		}
		else if(g_iPNMode == 1)
		{
			DisplayMenu(DisplayNote_Menu(client, iFlags, sPath), client, 0);
		}
		else if(g_iPNMode == 2)
		{
			DisplayNote_Chat(client, sPath);
			DisplayMenu(DisplayNote_Menu(client, iFlags, sPath), client, 0);
		}
		else
		{
			LogError("Cvar 'tognotes_pn_mode' must be set to either 0, 1, or 2. Current value (%i) is invalid!", g_iPNMode);
		}
	}
	return Plugin_Continue;
}

DisplayNote_Chat(client, String:sFile[])
{
	decl String:sBuffer[500], String:sHex[7];
	if(StrEqual(g_sChatHex, "", false))
	{
		Format(sHex, sizeof(sHex), "FF0000");
	}
	else
	{
		strcopy(sHex, sizeof(sHex), g_sChatHex);
	}
	new Handle:hFile = OpenFile(sFile, "r+");
	if(hFile != INVALID_HANDLE)
	{
		PrintToChat(client, "\x07%s=== %s ===", sHex, TAG);
		while(ReadFileLine(hFile, sBuffer, sizeof(sBuffer)))
		{
			new iBufferEnd = strlen(sBuffer) - 1;
			if (sBuffer[iBufferEnd] == '\n')
			{
				sBuffer[iBufferEnd] = '\0';
			}
			
			if(StrEqual(sBuffer, "", false))
			{
				Format(sBuffer, sizeof(sBuffer), " ");
			}
			
			if(StrContains(sBuffer, "{HEX}", false) != -1)
			{
				ReplaceString(sBuffer, sizeof(sBuffer), "{HEX}", "", true);
				strcopy(sHex, sizeof(sHex), sBuffer);
			}
			else
			{
				PrintToChat(client, "\x04- \x07%s%s", sHex, sBuffer);
			}
		}
		if(StrEqual(g_sChatHex, "", false))
		{
			Format(sHex, sizeof(sHex), "FF0000");
		}
		else
		{
			strcopy(sHex, sizeof(sHex), g_sChatHex);
		}
		PrintToChat(client, "\x07%s=== END NOTE ===", sHex, TAG);
	}
	if(hFile != INVALID_HANDLE)
	{
		CloseHandle(hFile);
	}
}

public Handle:DisplayNote_Menu(client, iDisableRemoveOption, String:sFile[])
{
	decl String:sBuffer[500];
	new Handle:hFile = OpenFile(sFile, "r+");
	new Handle:hMenu = CreateMenu(Handler_Menu);
	SetMenuTitle(hMenu, "TOG Notes");
	if(hFile != INVALID_HANDLE)
	{
		while(ReadFileLine(hFile, sBuffer, sizeof(sBuffer)))
		{
			new iBufferEnd = strlen(sBuffer) - 1;
			if (sBuffer[iBufferEnd] == '\n')
			{
				sBuffer[iBufferEnd] = '\0';
			}
			AddMenuItem(hMenu, "0", sBuffer, ITEMDRAW_DISABLED);
		}
	}
	AddMenuItem(hMenu, "0", "EXIT MENU (KEEP NOTICE)", _);
	if(!iDisableRemoveOption)
	{
		AddMenuItem(hMenu, "1", "REMOVE NOTIFICATION", _);
	}
	if(hFile != INVALID_HANDLE)
	{
		CloseHandle(hFile);
	}
	
	SetMenuExitBackButton(hMenu, true);
	
	return(hMenu);
}

public Handler_Menu(Handle:menu, MenuAction:action, client, param2)
{
	switch(action)
	{
		case MenuAction_End:
		{
			CloseHandle(menu);
		}
		case MenuAction_Cancel:
		{
			CloseHandle(menu);
		}
		case MenuAction_Select:
		{
			decl String:sTemp[32];
			GetMenuItem(menu, param2, sTemp, sizeof(sTemp));
			
			new iSelected;
			iSelected = StringToInt(sTemp);
			if(iSelected == 1)
			{
				RemovePNFile(client);
			}
			
			CloseHandle(menu);
		}
	}
	return;
}

RemovePNFile(client)
{
	decl String:sID[MAX_NAME_LENGTH];
	GetClientAuthString(client, sID, sizeof(sID));
	ReplaceString(sID, sizeof(sID), ":", "_");
	decl String:sTime[7];
	FormatTime(sTime, sizeof(sTime), "%m%d%y");
	decl String:sFromPath[PLATFORM_MAX_PATH], String:sToPath[PLATFORM_MAX_PATH];
	BuildPath(Path_SM, sFromPath, sizeof(sFromPath), "configs/tognotes/%s.txt", sID);
	BuildPath(Path_SM, sToPath, sizeof(sToPath), "configs/tognotes/removed/%s_%s.txt", sID, sTime);
	if(FileExists(sFromPath))
	{
		if(FileExists(sToPath))
		{
			new i = 2;
			while(FileExists(sToPath))
			{
				BuildPath(Path_SM, sToPath, sizeof(sToPath), "configs/tognotes/removed/%s_%s_%i.txt", sID, sTime, i);
				i++;
			}
		}
		new Handle:hFile = OpenFile(sFromPath, "r");
		new Handle:hFileTemp = OpenFile(sToPath, "w");
		
		if (hFile != INVALID_HANDLE)
		{
			decl String:sBuffer[1000];
			while(ReadFileLine(hFile, sBuffer, sizeof(sBuffer)))
			{
				new iBufferEnd = strlen(sBuffer) - 1;
				if (sBuffer[iBufferEnd] == '\n')
				{
					sBuffer[iBufferEnd] = '\0';
				}
				
				WriteFileLine(hFileTemp, sBuffer);
			}
		}
		
		if (hFile != INVALID_HANDLE)
		{
			CloseHandle(hFile);
		}
		if (hFileTemp != INVALID_HANDLE)
		{
			CloseHandle(hFileTemp);
		}
		DeleteFile(sFromPath);
		PrintToChat(client, "%s\x07%s Personal message has been removed!", TAG, g_sChatHex);
	}
	else
	{
		LogError("Unable to find file for moving: %s", sFromPath);
	}
}

bool:HasFlags(client, String:sFlags[])
{
	new AdminId:aid = GetUserAdmin(client);
	
	if(CheckCommandAccess(client, "sm_not_a_command", ADMFLAG_ROOT, true))
	{
		return true;
	}
	
	if(aid != INVALID_ADMIN_ID)
	{
		new iCount, iFound, flags = ReadFlagString(sFlags);
		for(new i = 0; i <= 20; i++)
		{
			if(flags & (1<<i))
			{
				iCount++;

				if(GetAdminFlag(aid, AdminFlag:i))
				{
					iFound++;
				}
			}
		}

		if(iCount == iFound)
		{
			return true;
		}
	}

	return false;
}

public bool:IsValidClient(client)
{
	if(!(1 <= client <= MaxClients) || !IsClientInGame(client) || IsFakeClient(client))
	{
		return false;
	}
	return true;
}